/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "private.h"



int send_poll(node n, const char *ip,  artnet_ttm_value_t ttm) {
	artnet_packet_t p ;

	if (n->state.mode != ARTNET_ON)
		return -1 ;

	if(n->state.node_type == ARTNET_SRV || n->state.node_type == ARTNET_RAW) {
		// check if caller specified an ip or wanted bcase
		if( ip != NULL) {
			if( ! inet_aton(ip, &p.to) ) {
				err_warn("ip conversion from %s failed", ip) ;
				return -1 ;
			}
		} else {
			p.to.s_addr = n->state.bcast_addr.s_addr ;
		}

		memcpy( &p.data.ap.id, ARTNET_STRING, ARTNET_STRING_SIZE) ;
		p.data.ap.opCode =  ARTNET_POLL;
		p.data.ap.verH = 0;
		p.data.ap.ver = ARTNET_VERSION;
		p.data.ap.ttm = ~ttm ;
		p.data.ap.pad = 0 ;

		p.length = sizeof(artnet_poll_t) ;
		net_send(n, &p) ;

		printf("Sending Poll\n") ;

	} else {
		err_warn("Not sending poll, not a server or raw device") ;
		return -1 ;
	}
	return 0 ;
}

/*
 *
 * @param n the node
 * @param response true if this reply is in response to a network packet
 * 				   false if this reply is due to the node changing it's conditions
 *
 */
int send_poll_reply(node n, int response) {
	artnet_packet_t reply ;
	int i ;
	
	if( ! response && n->state.mode == ARTNET_ON) {
		n->state.ar_count++ ;
	}

	reply.to = n->state.reply_addr ;
	reply.type = ARTNET_REPLY ;
	reply.length = sizeof(artnet_reply_t) ;

	// copy from a poll reply template
	memcpy(&reply.data, &n->ar_temp , sizeof(artnet_reply_t)  ) ; 
 
	for (i=0; i< ARTNET_MAX_PORTS; i++) {
		reply.data.ar.goodinput[i] = n->ports.in[i].port_status ;
		reply.data.ar.goodoutput[i] = n->ports.out[i].port_status ;
	}

	snprintf((char *) &reply.data.ar.nodereport,sizeof(reply.data.ar.nodereport) , "%04hx [%04i] libartnet" , n->state.report_code , n->state.ar_count)  ;

	return net_send(n, &reply) ; 
}



int send_tod_data(node n) {
	artnet_packet_t tod ;

	printf("sending tod data\n") ;

	tod.to = n->state.bcast_addr ;
	tod.type = ARTNET_TODDATA ;
	tod.length = sizeof(artnet_toddata_t) ;

	memset(&tod.data,0x00, tod.length) ;
	
	// set up the data
	memcpy( &tod.data.toddata.id, ARTNET_STRING, ARTNET_STRING_SIZE) ;
	tod.data.toddata.opCode = ARTNET_TODDATA;
	tod.data.toddata.verH = 0;
	tod.data.toddata.ver = ARTNET_VERSION;

	return net_send(n, &tod) ; 

}


int send_firmware_reply(node n, in_addr_t ip, artnet_firmware_status_code code) {
	artnet_packet_t p ;

	memset(&p, 0x0, sizeof(p) ) ;
		
	p.to.s_addr = ip ;
	p.length = sizeof(artnet_firmware_t) ;
	p.type = ARTNET_FIRMWAREREPLY ;
		
	// now build packet
	memcpy( &p.data.firmware.id, ARTNET_STRING, ARTNET_STRING_SIZE) ;
	p.data.firmware.opCode =  ARTNET_FIRMWAREREPLY;
	p.data.firmware.verH = 0;
	p.data.firmware.ver = ARTNET_VERSION;
	p.data.firmware.type = code ;

	return net_send(n, &p) ; 
}


int send_firmware_packet(node n, firmware_transfer_t *firm ) {
	artnet_packet_t p ;
	uint8_t type = 0;
	int data_len, max_len ;

	memset(&p, 0x0, sizeof(p) ) ;

	// max value of data_len is 1024 ;
	max_len = ARTNET_FIRMWARE_SIZE * sizeof(p.data.firmware.data[0]) ;

	// calculate length 
	data_len = firm->bytes_total - firm->bytes_current ;
	data_len = min(data_len , max_len) ;

	// work out type - 6 cases
	if( firm->ubea) {
		// ubea upload
		if(firm->bytes_current == 0) {
			// first
			type = ARTNET_FIRMWARE_UBEAFIRST ;
		} else if (data_len == max_len) {
			// cont
			type = ARTNET_FIRMWARE_UBEACONT ;
		} else if (data_len < max_len) {
			// last
			type = ARTNET_FIRMWARE_UBEALAST ;
		} else {
			// this should never happen, something has gone wrong
			printf("Attempting to send %d when the max is %d, very very bad...\n", data_len, max_len) ;
		}
	} else {
		// firmware upload
		if(firm->bytes_current == 0) {
			// first
			type = ARTNET_FIRMWARE_FIRMFIRST ;
		} else if (data_len == max_len) {
			// cont
			type = ARTNET_FIRMWARE_FIRMCONT ;
		} else if (data_len < max_len) {
			// last
			type = ARTNET_FIRMWARE_FIRMLAST ;
		} else {
			// this should never happen, something has gone wrong
			printf("Attempting to send %d when the max is %d, very very bad...\n", data_len, max_len) ;
		}
	}

	// set packet properties
	p.to.s_addr = firm->peer.s_addr ;
	p.length = sizeof(artnet_firmware_t) ;
	p.type = ARTNET_FIRMWAREMASTER ;
		
	// now build packet
	memcpy( &p.data.firmware.id, ARTNET_STRING, ARTNET_STRING_SIZE) ;
	p.data.firmware.opCode =  ARTNET_FIRMWAREMASTER;
	p.data.firmware.verH = 0;
	p.data.firmware.ver = ARTNET_VERSION;
	p.data.firmware.type =  type ;
	p.data.firmware.blockId = firm->expected_block ;

	int_to_bytes(firm->bytes_total / sizeof(uint16_t) , p.data.firmware.length) ;

	memcpy(&p.data.firmware.data, firm->data + (firm->bytes_current / sizeof(uint16_t) ), data_len) ;

	if ( net_send(n, &p) ) {
			// send failed
			return -1 ;
	} else {
		// update stats
		firm->bytes_current = firm->bytes_current + data_len ;
		firm->last_time = time(NULL) ;
		firm->expected_block++ ;
		// limit between 0 and 255 (only 8 bits wide)
		// we dont' actually need this cause it will be shorted when assigned above
		firm->expected_block %= UINT8_MAX ;
	}
	return 0;
}

